@@ -27,10 +27,12 @@ tsp_insertion <- function(x, type = "nearest", control = NULL){
     ## lenght(x) == 1
     choose1 <- function(x) if(length(x) > 1) sample(x, 1) else x
     
+    min_nonNA <- function(x) { x[is.na(x)] <- Inf; min(x) }
+    
     ## this is slower than which.min and which.max but works also
     ## correctly for only values Inf in x and breaks ties randomly 
-    choose1_min <- function(x) choose1(which(x == min(x)))
-    choose1_max <- function(x) choose1(which(x == max(x)))
+    choose1_min <- function(x) { x[is.na(x)] <- Inf; choose1(which(x == min(x))) }
+    choose1_max <- function(x) { x[is.na(x)] <- -Inf; choose1(which(x == max(x))) }
 
     types <- c("nearest", "farthest", "cheapest", "arbitrary")
     type_num <- pmatch(type, types)
@@ -75,7 +77,7 @@ tsp_insertion <- function(x, type = "nearest", control = NULL){
             }
             
             ds <- sapply(1:length(ks), FUN = 
-                function(i)  min(m[i, , drop = FALSE]))
+                function(i)  min_nonNA(m[i, , drop = FALSE]))
 
             winner_index <- crit(ds)
             k <- ks[winner_index] 
@@ -84,8 +86,8 @@ tsp_insertion <- function(x, type = "nearest", control = NULL){
         ## cheapest
         else if(type_num == 3) {
             winner_index <- choose1_min(sapply(ks, FUN =
-                    function(k) min(.Call("insertion_cost", x, order, k, 
-				    PACKAGE="TSP"))))
+                    function(k) min_nonNA(.Call("insertion_cost", x, order, k, 
+				    PACKAGE="TSP")) ))
             k <- ks[winner_index]
 
             ## we look for the optimal insertion place for k again later
